"""add_settlement_activity_and_status_fields

Revision ID: e981855d0418
Revises: manual_0002
Create Date: 2025-05-22 02:13:06.419914

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision: str = 'e981855d0418'
down_revision: Union[str, None] = 'manual_0002'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None

# Define Enum types for use in upgrade and downgrade
expense_split_status_enum = postgresql.ENUM('unpaid', 'partially_paid', 'paid', name='expensesplitstatusenum')
expense_overall_status_enum = postgresql.ENUM('unpaid', 'partially_paid', 'paid', name='expenseoverallstatusenum')

def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    
    # Create ENUM types
    expense_split_status_enum.create(op.get_bind(), checkfirst=True)
    expense_overall_status_enum.create(op.get_bind(), checkfirst=True)

    # Add 'overall_settlement_status' column to 'expenses' table
    op.add_column('expenses', sa.Column('overall_settlement_status', expense_overall_status_enum, server_default='unpaid', nullable=False))

    # Add 'status' and 'paid_at' columns to 'expense_splits' table
    op.add_column('expense_splits', sa.Column('status', expense_split_status_enum, server_default='unpaid', nullable=False))
    op.add_column('expense_splits', sa.Column('paid_at', sa.DateTime(timezone=True), nullable=True))

    # Create 'settlement_activities' table
    op.create_table('settlement_activities',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('expense_split_id', sa.Integer(), nullable=False),
        sa.Column('paid_by_user_id', sa.Integer(), nullable=False),
        sa.Column('paid_at', sa.DateTime(timezone=True), server_default=sa.text('now()'), nullable=False),
        sa.Column('amount_paid', sa.Numeric(precision=10, scale=2), nullable=False),
        sa.Column('created_by_user_id', sa.Integer(), nullable=False),
        sa.Column('created_at', sa.DateTime(timezone=True), server_default=sa.text('now()'), nullable=False),
        sa.Column('updated_at', sa.DateTime(timezone=True), server_default=sa.text('now()'), nullable=False), # Removed onupdate for initial creation
        sa.ForeignKeyConstraint(['created_by_user_id'], ['users.id'], ),
        sa.ForeignKeyConstraint(['expense_split_id'], ['expense_splits.id'], ),
        sa.ForeignKeyConstraint(['paid_by_user_id'], ['users.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_settlement_activity_created_by_user_id'), 'settlement_activities', ['created_by_user_id'], unique=False)
    op.create_index(op.f('ix_settlement_activity_expense_split_id'), 'settlement_activities', ['expense_split_id'], unique=False)
    op.create_index(op.f('ix_settlement_activity_paid_by_user_id'), 'settlement_activities', ['paid_by_user_id'], unique=False)
    
    # Manually add onupdate trigger for updated_at as Alembic doesn't handle it well for all DBs
    # For PostgreSQL, this is typically done via a trigger function.
    # However, for simplicity in this migration, we rely on the application layer to update this field.
    # Or, if using a database that supports it directly in Column definition (like some newer SQLAlch versions for certain backends):
    # op.alter_column('settlement_activities', 'updated_at', server_default=sa.text('now()'), onupdate=sa.text('now()'))
    # For now, the model has onupdate=func.now(), which SQLAlchemy ORM handles. The DDL here is for initial creation.

    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_settlement_activity_paid_by_user_id'), table_name='settlement_activities')
    op.drop_index(op.f('ix_settlement_activity_expense_split_id'), table_name='settlement_activities')
    op.drop_index(op.f('ix_settlement_activity_created_by_user_id'), table_name='settlement_activities')
    op.drop_table('settlement_activities')

    op.drop_column('expense_splits', 'paid_at')
    op.drop_column('expense_splits', 'status')

    op.drop_column('expenses', 'overall_settlement_status')

    # Drop ENUM types
    expense_split_status_enum.drop(op.get_bind(), checkfirst=False)
    expense_overall_status_enum.drop(op.get_bind(), checkfirst=False)
    # ### end Alembic commands ###
